// DCLinabox.js
//
// Core JavaScript for DCLinabox.
//
// Copyright (C) 2011,2012 Mark G Daniel <mark.daniel@wasd.vsm.com.au>
// Based on ShellInABox ... http://shellinabox.com/
// Copyright (C) 2008,2009 Markus Gutschke <markus@shellinabox.com>
//
// 28-APR-2012  MGD  v1.0.1, kludge for Firefox line height discrepencies
//                           DCLinaboxWxH configuration
// 04-DEC-2011  MGD  initial
var DCLinaboxVersion = "v1.0.1";

/////////////////////////
// configuration settings
/////////////////////////

// DO NOT MODIFY THESE, follow the instructions in CONFIGINABOX.JS

// language configuration
messageDefault = { NOTSUP : 'WebSocket not supported!', 
                   // buttons, etc.
                   CONNEC : 'CONNECT',
                   DISCON : 'DISCONNECT',
                   DISURE : 'DISCONNECT: Are you sure?',
                   PRINT  : 'Print',
                   // messages for connection status
                   FAILED : 'FAILED to connect',
                   NORESP : 'CONNECTED but no response',
                   BROKEN : 'CONNECTION broken ',
                   DISCED : 'DISCONNECTED'
                 };

getParameter('DCLinaboxMessage',messageDefault);

for (var key in messageDefault)
   if (typeof DCLinaboxMessage[key] == 'undefined')
      DCLinaboxMessage[key] = messageDefault[key];

// always connect via "wss:" (i.e. via SSL, true or false)
getParameter('DCLinaboxSecureAlways',true);

// connect immediately the page opens
getParameter('DCLinaboxImmediate',false);

// by default the [^] button is available on a standalone terminal
getParameter('DCLinaboxAnother',false);

// suppress scrollbar using 0 or set to number of lines in buffer (e.g. 500)
getParameter('DCLinaboxScroll',0);

// obscuring the application by using an alternate script name
getParameter('DCLinaboxScriptName','dclinabox');

// explicitly title the window
getParameter('DCLinaboxTitle','');

// get any explictly defined host name or default to the current host
getParameter('DCLinaboxHost',window.location.host);

// these are ShellInABox (vt100.js) configuration elements ...

getParameter('suppressAllAudio',true);
getParameter('linkifyURLs',1);
userCSSListDefault = [ [ 'White on Black', true, false ],
                       [ 'Black on White', false, true ],
                       [ 'Monochrome', true, false ],
                       [ 'Color Terminal', false, true ] ];
getParameter('userCSSList',userCSSListDefault);

// terminal size
getSizeParameter ();

var charWidth = 0;
var charHeight = 0;
var scrollWidth = 0;
var dclws = null;
var thisDCLinabox = null;
var vtterm = null;
var vtstatus = null;
var vtinabox = null;
var esc = String.fromCharCode(27);
// 0=unconnected,1=connecting,2=connected,3..n=data_rx,-1=manually_closed
var connectionStatus = 0;

// from a bookmarklet if no parent with openDCLinabox() available
var bookmarkletTerminal =
    !((opener && typeof opener.openDCLinabox != 'undefined') ||
      (parent && typeof parent.openDCLinabox != 'undefined'));

/////////////////////////////////////////
// instantiate the DCLinabox/VT100 object
/////////////////////////////////////////

// monkey see, monkey do

function extend(subClass, baseClass) {
  function inheritance() { }
  inheritance.prototype          = baseClass.prototype;
  subClass.prototype             = new inheritance();
  subClass.prototype.constructor = subClass;
  subClass.prototype.superClass  = baseClass.prototype;
};

function DCLinabox() {

   vtterm = document.getElementById ('vt100');
   vtstatus = document.getElementById ('vtstatus');
   vtinabox = document.getElementById ('vtinabox');

   // calculate a monospace character's width and height
   vtterm.innerHTML = '<div id="scrollor" ' +
      'style="width=10em;height=2em;overflow:scroll;"><br></div>' +
      '<table><tbody><td id="cursor" style="margin:0px;padding:0px;">' +
      '&nbsp;</td></tbody></table>';
   var cid = document.getElementById('cursor');

   var navualo = navigator.userAgent.toLowerCase();
   if (navualo.indexOf('firefox') > -1) {
      // Firefox (11.0 and 12.0 at least) seems to have a different (to Chrome)
      // line-height reported via .clientHeight, kludge an adjustment for this
      // on OS-X and Linux it's a single pixel
      var cch = cid.clientHeight - 1;
      // and on Windows it's three pixels!
      if (navualo.indexOf('windows') > -1) cch -= 2;
      vtterm.style.lineHeight = cch + 'px';
   }

   charWidth = cid.clientWidth;
   charHeight = cid.clientHeight;

   if (DCLinaboxScroll) {
      var scrollor = document.getElementById('scrollor');
      scrollWidth = scrollor.offsetWidth - scrollor.clientWidth;
   }

   // if it has come from a bookmarklet
   parseHash();

   // set the terminal elements' height and then width
   vtterm.style.height = (charHeight * DCLinaboxHeight) + 2 + 'px';
   vtstatus.style.height = charHeight + 5 + 'px';
   vtinabox.style.height = vtterm.offsetHeight +
                           vtstatus.offsetHeight + 'px';
   DCLinabox.prototype.DCLinaboxSetWidth(DCLinaboxWidth);

   if (opener) makeTerminal();

   this.superClass.constructor.call(this, vtterm);

   this.maxScrollbackLines = DCLinaboxScroll;
                                                                   
   thisDCLinabox = this;

   if ('WebSocket' in window) {
      if (DCLinaboxImmediate)
         webSocketOpen();
      else
         terminalStatus();
   }
   else
      terminalStatus(DCLinaboxNotSupportedMsg);
};
extend(DCLinabox, VT100);

// override the VT100.JS function
// transmit vt100.js key-press to PTD (via WebSocket)

DCLinabox.prototype.keysPressed = function(ch) {
  if (dclws)
     dclws.send(ch);
  else
  if (ch == esc)
     connTerm();
};

// override the VT100.JS function
// add our bell character to the VT status line

DCLinabox.prototype.beep = function() {

   bellChar(true);

   if (this.visualBell) {
      this.flashScreen();
   } else {
     try {
       this.beeper.Play();
     } catch (e) {
       try {
         this.beeper.src = 'beep.wav';
       } catch (e) {
       }
     }
   }
};

// override the VT100.JS function
// we'll never need to display that size

DCLinabox.prototype.showCurrentSize = function() {
};

// resize the terminal window

DCLinabox.prototype.DCLinaboxSetWidth = function(cols) {

   DCLinaboxWidth = cols;

   vtterm.style.width = vtstatus.style.width =
      (charWidth * DCLinaboxWidth) + 2 + scrollWidth + 'px';
   vtinabox.style.width = vtterm.offsetWidth + 'px';

   this.initializeElements();
   this.reset();
   terminalStatus();

   if (makeTerminalPopup)
      makeTerminal();
   else
      makeIframe();

   this.focusCursor();
   this.input.focus();
};

////////////////////////////////
// open the WebSocket connection
////////////////////////////////

function webSocketOpen () {

   if (!('WebSocket' in window)) {
      terminalStatus(DCLinaboxMessage.NOTSUP);
      return;
   }

   // with dynamic host list it may have changed since the page was generated!
   getParameter('DCLinaboxHost',window.location.host);

   if (DCLinaboxTitle.length) {
      if (window.parent &&
          window.frameElement &&
          window.parent.setDCLinaboxTitle)
         window.parent.setDCLinaboxTitle(DCLinaboxTitle);
      else
         setDCLinaboxTitle(DCLinaboxTitle);
   }
   else
      setDCLinaboxTitle(DCLinaboxTitle);

   if (DCLinaboxHost.substr(0,6) == 'wss://') {
      var URL = 'wss://';
      DCLinaboxHost = DCLinaboxHost.substr(6);
   }
   else
   if (DCLinaboxHost.substr(0,5) == 'ws://') {
      var URL = 'ws://';
      DCLinaboxHost = DCLinaboxHost.substr(5);
      if (DCLinaboxSecureAlways) URL = 'wss://';
   }
   else
   if (DCLinaboxSecureAlways)
      var URL = 'wss://';
   else
   if (window.location.protocol == 'https:')
      var URL = 'wss://';
   else
      var URL = 'ws://';

   var socketPort = window.location.port;

   if (DCLinaboxHost.indexOf(':') >= 0) {
      // includes a specific port
      socketPort = DCLinaboxHost.split(':')[1];
      DCLinaboxHost = DCLinaboxHost.split(':')[0];
      // if only the port was supplied
      if (DCLinaboxHost == '') DCLinaboxHost = window.location.host;
   }
   URL += DCLinaboxHost;

   if (socketPort.length && socketPort != 80 && socketPort != 443)
      URL += ':' + socketPort;

   // path is optional
   if (DCLinaboxScriptName.indexOf('/') == -1)
      URL += '/cgiplus-bin/' + DCLinaboxScriptName;
   else
      URL += DCLinaboxScriptName;

   connectionStatus = 1;

   try { dclws = new WebSocket(URL) }
   catch (err) { alert(err); }

   if (typeof dclws.protocol == 'undefined') {
      // WebSocket API/version is not recent enough
      dclws.close();
      dclws = null;
      connectionStatus = 0;
      terminalStatus(DCLinaboxMessage.NOTSUP);
      return;
   }

   // WebSocket open

   dclws.onopen = function(evt) {
      connectionStatus = 2;
      DCLinaboxImmediate = true;
      window.onbeforeunload = function() { return dclws.close() };
      vtterm.style.backgroundColor = 'white';
      dclws.send('\r');
      terminalStatus();
      thisDCLinabox.focusCursor();
      thisDCLinabox.input.focus();
   };

   // WebSocket close

   dclws.onclose = function(evt) {
      var msg = null;
      var reason = '';
      if (typeof evt != 'undefined') {
         var code = evt.code;
         var reason = ' (' + evt.code
         if (evt.reason.length) reason += ' ' + evt.reason;
         reason += ')';
      }
      switch (connectionStatus) {
         case -1 : msg = DCLinaboxMessage.DISCED ; break;
         case  0 : msg = '?'; break;
         case  1 : msg = DCLinaboxMessage.FAILED + reason; break;
         case  2 : msg = DCLinaboxMessage.NORESP + reason; break;
         default : var dt = new Date;
                   msg = DCLinaboxMessage.BROKEN +
                         '<span style="font-size:80%;">' +
                         dt.toDateString() + ' ' +
                         dt.toTimeString().substr(0,8) +
                         '</span>' + reason;
                   break;
      }
      connectionStatus = 0;
      vtterm.style.backgroundColor = 'whitesmoke';
      terminalStatus(msg);
      dclws = null;
   };

   // WebSocket data from PTD

   dclws.onmessage = function (evt) { 
      var termResponse = thisDCLinabox.vt100(evt.data);
      if (termResponse.length) dclws.send(termResponse);
      connectionStatus++;
   };
}

///////////////////////////
// get parameters from hash
///////////////////////////

function parseHash () {

   if (!window.location.hash.length) return;

   var hash = window.location.hash.substr(1);
   if (hash.substr(0,6) == 'wss://') {
      var protocol = 'wss://';
      hash = hash.substr(6);
   }
   else
   if (hash.substr(0,5) == 'ws://') {
      var protocol = 'ws://';
      hash = hash.substr(5);
   }
   else {
      alert('ERROR:'+hash);
      return;
   }

   var WxH = null;
   var sizeat = hash.indexOf(';');
   if (sizeat > 0) {
      WxH = hash.substr(sizeat+1)
      hash = hash.substr(0,sizeat);
   }

   var pathat = hash.indexOf('/');
   if (pathat > 0) {
      var host = hash.substr(0,pathat)
      var script = hash.substr(pathat);
   }
   else {
      alert('ERROR:'+hash);
      return;
   }

   DCLinaboxHost = protocol + host;
   DCLinaboxScriptName = script;
   if (WxH) {
      DCLinaboxWxH = WxH;
      getSizeParameter ();
   }
}

//////////////////////////////////
// get the terminal size parameter
//////////////////////////////////

function getSizeParameter () {

   // width x height
   getParameter('DCLinaboxWxH',null);
   if (DCLinaboxWxH != null) {
      var WxH = DCLinaboxWxH.split('x');
      if (WxH.length == 2) {
         DCLinaboxWidth = parseInt(WxH[0]);
         DCLinaboxHeight = parseInt(WxH[1]);
      }
   }

   // opening terminal width and height
   getParameter('DCLinaboxWidth',80);
   if (DCLinaboxWidth != 80 && DCLinaboxWidth != 132) DCLinaboxWidth = 80;
   getParameter('DCLinaboxHeight',24);
   if (DCLinaboxHeight < 12 || DCLinaboxHeight > 96) DCLinaboxHeight = 24;
}

////////////////////////////////////
// make the popup a terminal window
////////////////////////////////////

var makeTerminalSnap = null;
var makeTerminalPopup = false;

function makeTerminal () {

   makeTerminalPopup = true;
   DCLinaboxImmediate = true;
   DCLinaboxAnother = true;
   document.body.style.margin = 0;
   document.body.style.padding = 0;
   vtinabox.style.margin = 0;

   if (navigator.appName=='Netscape') {
      var width = window.innerWidth;
      var height = window.innerHeight;
   } else {
      var width = document.body.clientWidth; 
      var height = document.body.clientHeight;
   }
   width = parseInt(vtinabox.style.width) - width;
   height = parseInt(vtinabox.style.height) - height;
   if (DCLinaboxScroll) { width++; height++; }
   window.resizeBy(width,height);

   window.onresize = function () {
      clearTimeout(makeTerminalSnap);
      makeTerminalSnap = setTimeout('makeTerminalSnap=null;makeTerminal()',500);
      return true;
   };
}

/////////////////////////////////////////////
// resize a parent IFRAME to fit the terminal
/////////////////////////////////////////////

function makeIframe () {

   if (!(window.parent &&
         window.frameElement &&
         window.parent.resizeDCLinaboxIframe)) return;
         
   document.body.style.margin = 0;
   document.body.style.padding = 0;
   vtinabox.style.margin = 0;

   var width = parseInt(vtinabox.style.width) + 10;
   var height = parseInt(vtinabox.style.height) + 10;
   window.parent.resizeDCLinaboxIframe(window.frameElement,width,height);
}

//////////////////
// terminal status
//////////////////

terminalStatusOpen1 = null;

function terminalStatus (msg) {

   var status = '&nbsp;';
   if (connectionStatus)
      status += '<span id="vtbtn" onclick="discTerm()">&nbsp;' +
                DCLinaboxMessage.DISCON + '&nbsp;</span>';
   else
      status += '<span id="vtbtn" onclick="connTerm()">&nbsp;' +
                DCLinaboxMessage.CONNEC + '&nbsp;</span>';
   if (DCLinaboxAnother)
      status += '<span id="vtbtn" onclick="openAnotherDCLinabox()' +
                '">&nbsp;^&nbsp;</span>';
   status += '<span id="vtbtn" style="margin-left:5px;" ' +
             'onclick="thisDCLinabox.vt100(esc+\'[i\')">&nbsp;' +
             DCLinaboxMessage.PRINT + '&nbsp;</span>';
   if (typeof msg != 'undefined' && msg != null)
      status += '<span>&nbsp;' + msg + '&nbsp;</span>';
   status += '&nbsp;<span id="vtbell">&nbsp;</span>&nbsp;';
   status += '<span id="vanity1">DCLinabox<span id="vanity2">' +
             DCLinaboxVersion + '</span></span>';
   vtstatus.innerHTML = status;
}

// connect button clicked

function connTerm () {
   thisDCLinabox.initializeElements();
   thisDCLinabox.reset();
   terminalStatus();
   webSocketOpen ();
}

// disconnect button clicked

function discTerm () {
   if (confirm(DCLinaboxMessage.DISURE)) {
      connectionStatus = -1;
      dclws.close();
   }
}

// [^] button clicked

function openAnotherDCLinabox () {

   var specs = 'toolbar=0,location=0,directories=0,status=0,menubar=0,' +
               'scrollbars=0,resizable=0,copyhistory=0,width=100,height=100';

   window.open(window.location.href,'_blank',specs);
}

// set the title of a standalone terminal
// this function can also be found in OPENINABOX.JS

function setDCLinaboxTitle (title) {
   if (typeof title == 'undefined' ||
       !title.length ||
       title.substr(0,1) == '?')
      title = 'DCLinabox: ' + DCLinaboxHost;
   document.getElementsByTagName('title')[0].innerHTML = title;
}

///////////////////////////////////////////
// "He's deaf. The bells have made him so."
///////////////////////////////////////////

var vtbell = null;
var bellCharCount = 0;
var utf237e = null;

function bellChar (ding) {
   if (!vtbell) vtbell = document.getElementById('vtbell');
   if (!utf237e) utf237e = '<img style="vertical-align:text-top; height:' +
                           charHeight + 'px;" src="utf237e.png">';
   if (ding) {
      if (bellCharCount > 10) return;
      bellCharCount++;
      setTimeout('bellChar(false)',900);
   }
   else {
      bellCharCount--;
   }
   var bellCharVisual = '';
   for (var cnt = bellCharCount; cnt; cnt--) bellCharVisual += utf237e;
   vtbell.innerHTML = bellCharVisual;
}

///////////////////
// inline execution
///////////////////

// create a style element and add some inline style

function addInlineStyle () {

   var style = '';

   style += '#vt100 { min-height: 16px; margin-bottom:0; ' +
            'height:4em; width:24em; } ';

   style += '#vtstatus { min-height: 16px; font-family: "DejaVu Sans Mono", ' +
            '"Everson Mono", FreeMono, "Andale Mono", "Lucida Console", ' +
            'monospace; font-size:90%; letter-spacing: -1px; margin-top:0; ' +
            'padding-top: 5px; border-top-width:0; height:1em; width:24em; ' +
            'overflow: hidden; white-space: nowrap; } ';

   style += '#vanity1 { position:absolute; right:0; margin:0 2em 0 2em; ' +
            'font-family:futura, helvetica, arial, sans; font-size:90%; ' +
            'line-height:80%; text-align:left; letter-spacing:0px; } ';

   style += '#vanity2 { display:block; text-align:center; font-size:80%; } ';

   style += '#vtbell { vertical-align:text-top; } ';

   style += '#vtbtn { margin: 1px 3px 1px 3px; padding: 1px 1px 1px 1px; } ';

   if (typeof DCLinaboxScroll == "undefined" || !DCLinaboxScroll)
      style += '#vt100 #scrollable { overflow: hidden; }';

   // comment from original ShellInABox DEMO.HTML on why this was included

   // We would like to hide overflowing lines as this can lead to
   // visually jarring results if the browser substitutes oversized
   // Unicode characters from different fonts. Unfortunately, a bug
   // in Firefox prevents it from allowing multi-line text
   // selections whenever we change the "overflow" style. So, only
   // do so for non-Netscape browsers.

   if (typeof navigator.appName == 'undefined' ||
       navigator.appName != 'Netscape')
     style += '#vt100 #console div, #vt100 #alt_console div ' +
              '{ overflow: hidden; }';

   var styleobj = document.createElement('style');
   styleobj.setAttribute('type','text/css');
   styleobj.innerHTML = style;
   document.getElementsByTagName('head')[0].appendChild(styleobj);
}

addInlineStyle();

window.onload = function () {
   setTimeout('new DCLinabox()',100);            
}

// end
